# -*- coding: utf-8 -*-
"""
Automatic tests for python-ldap

See https://www.python-ldap.org/ for details.
"""

from __future__ import absolute_import

from . import t_bind
from . import t_cext
from . import t_cidict
from . import t_ldap_dn
from . import t_ldap_filter
from . import t_ldap_functions
from . import t_ldap_modlist
from . import t_ldap_schema_tokenizer
from . import t_ldapurl
from . import t_ldif
from . import t_ldapobject
from . import t_edit
from . import t_ldap_schema_subentry
from . import t_untested_mods
from . import t_ldap_controls_libldap
from . import t_ldap_options
